@@ -1,10 +1,11 @@
 .extractOpticut <-
 function (object, which=NULL, boot=FALSE,
-internal=TRUE, best=TRUE, ...)
+internal=TRUE, best=TRUE, Z=NULL, ...)
 {
     if (is.null(which))
         which <- names(object$species)
-    bp <- bestpart(object)
+    if (is.null(Z))
+        bp <- bestpart(object)
     spp <- names(object$species)
     names(spp) <- spp
     spp <- spp[which]
@@ -20,16 +21,31 @@ internal=TRUE, best=TRUE, ...)
         if (internal) {
             if (!best)
                 stop("use best=TRUE when internal=TRUE")
+            if (is.null(Z)) {
+                ZZ <- bp[j,i]
+            } else {
+                if (is.null(dim(Z))) {
+                    if (is.factor(Z)) {
+                        ZZ <- model.matrix(~Z, data.frame(Z=Z))[j,-1L,drop=FALSE]
+                    } else {
+                        ZZ <- Z[j]
+                    }
+                } else {
+                    ZZ <- Z[j,,drop=FALSE]
+                }
+            }
             out[[i]] <- .opticut1(
                 Y=object$Y[j,i],
-                X=object$X[j,],
-                Z1=bp[j,i],
+                X=object$X[j,,drop=FALSE],
+                Z1=ZZ,
                 dist=object$dist, ...)
         } else {
+            if (!is.null(Z))
+                stop("Z used only for best=TRUE & internal=TRUE")
             if (best) {
                 out[[i]] <- opticut1(
                     Y=object$Y[j,i,drop=TRUE],
-                    X=object$X[j,],
+                    X=object$X[j,,drop=FALSE],
                     Z=bp[j,i,drop=FALSE],
                     dist=object$dist, ...)
             } else {
@@ -41,7 +57,7 @@ internal=TRUE, best=TRUE, ...)
                 }
                 out[[i]] <- opticut1(
                     Y=object$Y[j,i,drop=TRUE],
-                    X=object$X[j,],
+                    X=object$X[j,,drop=FALSE],
                     Z=zz,
                     dist=object$dist, ...)
             }
